#!/usr/bin/env python

# exit codes
NOT_OK = 1
OK = 0

# modules for OPUS internal poller
import sys                  # access to command-line args
import string               # string functions
import opusutil             # utilites like PrintMsg
import re                   # regular expression handling
import traceback            # printing of stack traces 

# python modules for this application
import os
import getopt

# local modules for this application
import datasources
import hst_missionquery
import hst_archdbquery
import kwdbquery
import xmlReferenceFileDefs
import siname
import base_poller

# constants
ALL_VALUES = 1      # includes db SQL for all ref files, whether changed or not

# exceptions
class MissingIORonCommandLine(Exception):
  pass

class NarrowOfOrbObjectFailed(Exception):
  pass

class FailedToRegisterPipelineApp(Exception):
  pass

class UsageError(Exception):
  pass

class NotFitsOrDbInput(Exception):
  pass

class MissingRequiredFiles(Exception):
  pass

class FailedResourceInit(Exception):
  pass

class PollingFailed(Exception):
  pass

class HaltEvent(Exception):
  pass

class DatasetNotFoundInOSF(Exception):
  pass

class Asn_table_clear(base_poller.InternalPoller):
  """
########################################################################
Class: Asn_table_clear

Description:
------------
This is a tool for use during STIS DMS association creation
for clearing the collect_date field of an existing STIS association,
and setting the asn_members.member_status values to 'U' (uncollected).
It is meant to be run against each wavecal_months/YYYYDDD.wavecal_old2new
file in preparation for creating the new ASNs described in that file.

03/28/07  57374  MSwam   first version
########################################################################
  """
  def __init__(self):
    # initialize members
    self._inpath = None
    self._db_retry_interval = 3600  # in seconds
    self._opus_dbobj = None
    self._opus_server = os.environ['OPUS_SERVER']
    self._opus_db     = os.environ['OPUS_DB']

  def initialize(self):
    #
    # call superclass constructor
    base_poller.InternalPoller.__init__(self)

  #--------------------------------------------------------------------------
  def usage(self):
    opusutil.PrintMsg('F','Interactive Usage: asn_table_clear.py [-h] input_old2new_prefix')
    raise UsageError

  #--------------------------------------------------------------------------
  #
  # Name: interactive
  #
  # Description: Performs command line parsing and set-up for
  #              processing in interactive mode.  Runs the
  #              task for the command-line arguments given.
  #
  # Inputs: global sys.argv (command line arguments)
  #          
  # Notes: Reads reference file definitions from an XML file.
  #        Will EXIT (status=1) if incorrect command line arguments
  #        are found.  This is a FATAL error.
  #
  # History:
  #   date    opr     who     reason
  # -------- -----  --------  --------------------------------------
  # 09/23/02 xxxxx  MSwam     first version
  #
  #--------------------------------------------------------------------------
  def interactive(self):
    #
    # parse command line arguments
    try:
      opts, args = getopt.getopt(sys.argv[1:], "h", 
                   ["help"])
    except getopt.GetoptError:
      # print help information and raise exception
      self.usage()

    # initialize
    dataset = None

    # parse options
    for flag, value in opts:
      if flag in ("-h", "--help"):
        self.usage()

    # verify required args
    if len(args) != 1:
      self.usage()
    else:
      inputfile_prefix = args[0]

    # connect to OPUS db
    self._opus_dbobj = hst_missionquery.HST_Mission_db(self._opus_server,
                                         self._opus_db,
                                         self._db_retry_interval)
    
    # set the directory for file operations to the current directory
    self._inpath = os.getcwd()

    # run the computations
    self.do_asn_table_clear(inputfile_prefix)

    # close the OPUS database connection
    self._opus_dbobj.close()

    return

  #-------------------------------------------------------------------------
  # 
  # Name: pipeline_init
  #
  # Description: Initialization for pipeline mode, including reading
  #              XML file of reference file specs and resource file
  #              parameters for controlling the task.
  #
  # History:
  # -------- -----  --------  --------------------------------------
  # 09/23/02 xxxxx  MSwam     first version
  #
  #-------------------------------------------------------------------------
  def pipeline_init(self):
    # obtain resource items in a dictionary and assign to globals
    #
    try:
      resource_items = base_poller.InternalPoller.get_resource_items(self,
                            ["INPATH",
                             "DB_RETRY_INTERVAL",
                             "OPUS_SERVER",
                             "OPUS_DB"] )
    except:
      opusutil.PrintMsg("E","failed to get required resource entry")
      raise
    #
    opusutil.PrintMsg("I","Resource items read.")
    self._inpath = resource_items['INPATH']
    self._db_retry_interval = int(resource_items['DB_RETRY_INTERVAL'])
    self._opus_server = resource_items['OPUS_SERVER']
    self._opus_db = resource_items['OPUS_DB']
    #
    # connect to the OPUS database
    self._opus_dbobj = hst_missionquery.HST_Mission_db(self._opus_server,
                                         self._opus_db,
                                         self._db_retry_interval)
    return

  #-------------------------------------------------------------------------
  # 
  # Name: do_asn_table_clear
  #
  # Description: Computes the best reference files for a dataset.
  #
  # Input:
  # inputfile_prefix - prefix of "old2new" input file listing asns and members
  #
  # History:
  # -------- -----  --------  --------------------------------------
  # 09/23/02 xxxxx  MSwam     first version
  #
  #-------------------------------------------------------------------------
  def do_asn_table_clear(self, inputfile_prefix):
    #
    inputfilename = inputfile_prefix + ".wavecal_old2new"
    for line in open(inputfilename, 'r').readlines():
       parts = line.split()  # split on whitespace
       if len(parts[1])==9:
          # this line results in generation of an asn_association update
          querytxt = ("UPDATE asn_association SET "+
                "collect_date = ' ' "+
                "WHERE association_id = '"+parts[0]+"' ")
          #
          opusutil.PrintMsg("I","attempting: "+querytxt)
          self._opus_dbobj._DB.query(querytxt)
          self._opus_dbobj._DB.executeUpdate()
       elif len(parts[1])==8:
          # this line results in generation of an asn_members update
          ppp = parts[1][1:4]
          ss  = parts[1][4:6]
          oo  = parts[1][6:8]
          querytxt = ("UPDATE asn_members SET "+
                "member_status = 'U' "+
                "WHERE association_id = '"+parts[0]+"' and "+
                "program_id = '"+ppp+"' and "+
                "obset_id   = '"+ss+"' and "+
                "member_num = '"+oo+"' and "+
                "member_status = 'C'")
          #
          opusutil.PrintMsg("I","attempting: "+querytxt)
          self._opus_dbobj._DB.query(querytxt)
          self._opus_dbobj._DB.executeUpdate()
       else:
          raise UnexpectedInputFormat
    return

  #-------------------------------------------------------------------------
  # Name: processHaltEvent
  #
  # Description: Handle a HALT event (override base class).
  #
  # History:
  # -------- -----  --------  --------------------------------------
  # 02/25/05 51433  MSwam     use base poller class
  #
  #-------------------------------------------------------------------------
  def processHaltEvent(self, event):
    #
    opusutil.PrintMsg("I","Received HALT event. Stopping.")
    self._opus_dbobj.close()
    raise HaltEvent


  #-------------------------------------------------------------------------
  # Name: processOsfEvent
  #
  # Description: Handle OSF events in pipeline mode.
  #
  # History:
  # -------- -----  --------  --------------------------------------
  # 09/23/02 xxxxx  MSwam     first version
  # 08/06/04 49860  Sontag    Changed an IDL field name
  # 12/17/04 51433  MSwam     Use proper IGNEVT name
  # 02/25/05 51433  MSwam     Use base_poller base class
  #
  #-------------------------------------------------------------------------
  def processOsfEvent(self, event):
    # get trigger info
    dataset, data_id = self.getDatasetDataidFromOsfEvent(event)

    try:
      opusutil.OpenTrl(self._trlpath + os.sep + dataset + ".trx")
    except:
      # any exception marks the OSF with OSF_ERROR
      traceback.print_exc()
      #
      # an error occurred during trailer opening, set the OSF to failed
      try:
        opusutil.PrintMsg("E","closing event with OSF_ERROR")
        self._papp.closeEvent("OSF_ERROR", event.eid)
      except:
        traceback.print_exc()
        opusutil.PrintMsg("E","Failed to close OSF_ERROR event")
      return

    opusutil.PrintMsg("I","----- REF started for: "+dataset+" -----")
    try:
      self.do_asn_table_clear(dataset)
    except:
      # any other exception marks the OSF with OSF_ERROR
      traceback.print_exc()
      #
      # an error occurred during processing, set the OSF
      try:
        opusutil.PrintMsg("E","closing event with OSF_ERROR")
        self._papp.closeEvent("OSF_ERROR", event.eid)
      except:
        traceback.print_exc()
        opusutil.PrintMsg("E","Failed to close OSF_ERROR event")
      opusutil.PrintMsg("I","----- REF ended for: "+dataset+" -----")
      opusutil.CloseTrl()
      return

    try:
      self._papp.closeEvent("OSF_NORMAL", event.eid)
    except:
      traceback.print_exc()
      opusutil.PrintMsg("E","Failed to close OSF_NORMAL event")
    opusutil.PrintMsg("I","----- REF ended for: "+dataset+" -----")
    opusutil.CloseTrl()

#-------------------------------------------------------------------------
# The main routine. Handles interactive and pipeline mode.
#
# History:
# -------- -----  --------  --------------------------------------
# 02/25/05 51433  MSwam     first version
#
#-------------------------------------------------------------------------
#
# initialize
try:
  poller = Asn_table_clear()
  poller.initialize()
  poller.pipeline_init()
#
except base_poller.InteractiveMode:
  opusutil.PrintMsg("I","Running command-line mode.")
  try:
    poller.interactive()
  except UsageError:
    sys.exit(NOT_OK)
  except:
    traceback.print_exc()
    opusutil.PrintMsg("F","Command-line mode Failed.")
    sys.exit(NOT_OK)
  sys.exit(OK)
#
except:
  traceback.print_exc()
  opusutil.PrintMsg("E","initialization failed")
  sys.exit(NOT_OK)

opusutil.PrintMsg("I","Pipeline set-up complete.")
try:
  poller.run()
except HaltEvent:
  sys.exit(OK)
except:
  traceback.print_exc()
  opusutil.PrintMsg("E","pipeline_process failed")
  sys.exit(NOT_OK)

